Package: efdm
Title: Simulate Forest Resources with the European Forestry Dynamics
        Model
Version: 0.2.3
Authors@R: c(
    person("Mikko", "Kuronen", email="mikko.kuronen@luke.fi", role=c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8089-7895")),
    person("Minna", "Räty", email="minna.raty@luke.fi", role="aut",
           comment = c(ORCID = "0000-0001-9898-8712")))
Description: An implementation of European Forestry Dynamics Model (EFDM) and
    an estimation algorithm for the transition probabilities.
    The EFDM is a large-scale forest model that simulates the development of
    the forest and estimates volume of wood harvested for any given forested
    area. This estimate can be broken down by, for example, species, site
    quality, management regime and ownership category.
    See Packalen et al. (2015) <doi:10.2788/153990>.
URL: https://github.com/mikkoku/efdm
BugReports: https://github.com/mikkoku/efdm/issues
License: GPL-2
Encoding: UTF-8
LazyData: true
Imports: stats, utils, data.table
Suggests: dplyr, ggplot2, gridExtra, knitr, rmarkdown, sf, testthat
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-04-24 05:09:23 UTC; 03052156
Author: Mikko Kuronen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8089-7895>),
  Minna Räty [aut] (ORCID: <https://orcid.org/0000-0001-9898-8712>)
Maintainer: Mikko Kuronen <mikko.kuronen@luke.fi>
Repository: CRAN
Date/Publication: 2026-04-27 05:30:03 UTC
