# Generated by roxygen2: do not edit by hand

export("transprobs<-")
export(define_activity)
export(estimatetransprobs)
export(prior_ff)
export(prior_grow)
export(runEFDM)
export(transprobs)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(stats,aggregate)
importFrom(utils,head)
