# efdm
An implementation of European Forestry Dynamics Model (EFDM) and
    an estimation algorithm for the transition probabilities.
    The EFDM is a large-scale forest model that simulates the development of
    the forest and estimates volume of wood harvested for any given forested
    area. This estimate can be broken down by, for example, species, site
    quality, management regime and ownership category.

The European Forestry Dynamics Model: Concept, design and results of first case studies, Packalen et al. DOI:[10.2788/153990](http://dx.doi.org/10.2788/153990)

# Installation

The package is available from CRAN [https://cran.r-project.org/package=efdm].
```R
install.packages("efdm")
vignette("example")
```

# Vignette

Vignette is also available outside R.

[Scenario example](https://mikkoku.github.io/efdm/vignettes/example.html)
